# Diplomacy Agents
This code provides an implementation of an A2C agent and 
a LOLA agent adapted to play Diplomacy.

## Command line arguments
```bash
train.py
    --game_config_path A path to a file which specifies properties of the game and the agents who will play a game of Diplomacy
run_tournament.py
    --tournament_config_path The path to the tournament config file
save_model.py  
    --checkpoint_path Path to the model checkpoint. Only diplomacy policy networks are supported
    --save_path Where to save the model
    --game_config_path A path to a file which specifies properties of the game and the agents which have been used to create the checkpoint
calculate_log_stats.py
    --dir_path A path to a directory containing log files
    --calculate_tournament_statistics Whether to plot tournament statistics
      (default: 'False')
    --tournament_map_name The Diplomacy map used in the tournament
      (default 'pure')
    --include_x_power_support_stats Whether to include statistics on cross power support. The calculation of the cross power statistics may take a while to finish
      (default 'False')
    --plot_validation_tournament_statistics Whether to plot validation tournament statistics
      (default 'False')
     --player_to_plot_stats_for Which player's statistics to plot. In the plot the player name is replaced with the rl algorithm that was used to train the agent, given that the name of the algorithm is part of the dir path. Supported rl algorithms (a2c and lola)
      (default "player1")
```

## Training the agents

Example of running a training session:

```python train.py --game_config_path configs/diplomacy_pure_map_config.json```

Example config files can be found in the configs folder.

## Running a tournament

Example of running a tournament:

```python train.py --tournament_config_path configs/diplomacy_tournament_config.json``` 

Example config files can be found in the configs folder.

## Convert checkpoints to a saved model

Example of converting checkpoints to a saved model:

```python save_model.py --checkpoint_path checkpoints/timestamp/policy/checkpoint-x --save_path saved_model/example --game_config_path configs/diplomacy_pure_map_config.json```

Example config files can be found in the configs folder.

## Calculate statistics from a log file

Example of calculating statistics from a log file:

```python calculate_log_stats.py --dir_path logs/timestamp/ --calculate_tournament_statsitics```

## Testing

To run all the tests in the application:

```python -m unittest discover -p *_test.py```

## Trained models

The trained A2C and LOLA agents can be found in the saved_models folder

