from absl import flags, app

from globals import Time
from statistics.diplomacy_statistics import calculate_and_print_log_stats, calculate_and_plot_training_log_stats

FLAGS = flags.FLAGS
flags.DEFINE_string("dir_path", None, "A path to a directory containing log files")
flags.DEFINE_bool("calculate_tournament_statistics", False, "Whether to plot tournament statistics")
flags.DEFINE_string("tournament_map_name", "pure", "The Diplomacy map used in the tournament")
flags.DEFINE_bool("include_x_power_support_stats", False,
                  "Whether to include statistics on cross power support. The calculation of the cross power statistics "
                  "may take a while to finish")
flags.DEFINE_bool("plot_validation_tournament_statistics", False, "Whether to plot validation tournament statistics")
flags.DEFINE_string("player_to_plot_stats_for", "player1", "Which player's statistics to plot. In the "
                                                           "plot the player name is replaced with the rl algorithm "
                                                           "that was used to train the agent, "
                                                           "given that the name of the algorithm is "
                                                           "part of the dir path. Supported rl algorithms (a2c and "
                                                           "lola)")


def main(argv):
    Time.set_start_time()
    if FLAGS.calculate_tournament_statistics:
        calculate_and_print_log_stats(FLAGS.dir_path, FLAGS.tournament_map_name, FLAGS.include_x_power_support_stats)
    if FLAGS.plot_validation_tournament_statistics:
        calculate_and_plot_training_log_stats(FLAGS.dir_path)


if __name__ == "__main__":
    app.run(main)
