from absl import flags, app
import os
import keras.models

from agents import AgentFactory
from callbacks.callbacks import CallbackFactory
from globals import Time
from tournament.rl_tournament import TournamentFactory
from games.games import GameFactory
from utils.utils import get_config

os.environ["TF_FORCE_GPU_ALLOW_GROWTH"] = "true"
os.environ["TF_GPU_ALLOCATOR"] = "cuda_malloc_async"

FLAGS = flags.FLAGS
flags.DEFINE_string("tournament_config_path", None, "Path to tournament config file")


def main(argv):
    Time.set_start_time()
    config = get_config(FLAGS.tournament_config_path)
    run_tournament(config)


def run_tournament(config):
    agents = []
    for agent_config in config["agents"]:
        agents += create_agent(**agent_config)
    callback_list = CallbackFactory.create_game_callbacks(config["game"]["name"],
                                                          file_name="tournament_log.csv")
    game = GameFactory.create_game(agents, callback_list, **config["game"])
    callback_list = CallbackFactory.create_tournament_callbacks("DEFAULT")
    tournament = TournamentFactory.create_tournament("DEFAULT", game, callback_list, **config["tournament"])
    tournament.play_games()


def create_agent(**agent_configs):
    pnet = None
    if agent_configs["saved_model_path"]:
        pnet = keras.models.load_model(agent_configs["saved_model_path"], compile=False)
    return [AgentFactory.create_agent(agent_type="A2C",
                                      critic_type=None,
                                      pnet=pnet,
                                      vnet=None,
                                      gamma=1.,
                                      **agent_configs) for _ in range(agent_configs["exemplars"])]


if __name__ == "__main__":
    app.run(main)
