from absl import flags, app

from callbacks import CallbackFactory
from globals import Time
from utils.utils import get_config
from games.games import GameFactory
from neural_networks.networks import NeuralNetworkFactory

FLAGS = flags.FLAGS
flags.DEFINE_string("checkpoint_path", None,
                    "Path to the model checkpoint. Only diplomacy policy networks are supported")
flags.DEFINE_string("save_path", None, "Where to save the model")
flags.DEFINE_string("game_config_path", None,
                    "A path to a file which specifies properties of the game and the agents which have been used to create the checkpoint")


def main(argv):
    Time.set_start_time()
    config = get_config(FLAGS.game_config_path)
    create_saved_model_from_checkpoints(FLAGS.checkpoint_path, FLAGS.save_path, config)


def create_saved_model_from_checkpoints(checkpoint_path, save_path, config):
    """
    Creates a saved_model from stored checkpoints. Only Diplomacy policy networks are supported
    :param checkpoint_path: The path to the stored checkpoints
    :param save_path: Where to save the model
    :param config: Config used when checkpoints were created
    :return:
    """
    pnet, vnet = NeuralNetworkFactory.create_neural_networks(config["game"]["name"],
                                                             config["neural_network_parameters"],
                                                             map_name=config["game"]["map_name"])
    players = [i for i in range(len(config["agents"]))]
    callback_list = CallbackFactory.create_tournament_callbacks("DEFAULT")
    game = GameFactory.create_game(players, callback_list, **config["game"])
    players = game.get_players()
    state = game.reset()
    available_actions = game.get_available_actions(players[0])
    pnet([state, available_actions])
    pnet.load_weights(checkpoint_path)
    pnet.save(save_path)


if __name__ == "__main__":
    app.run(main)
