import json
import os

import numpy as np
import tensorflow as tf
from pathlib import Path
from copy import deepcopy


class TrainableWeightsReshaper:

    def __init__(self, trainable_weights):
        self.shapes = self.__get_dimensions(trainable_weights)

    @staticmethod
    def __get_dimensions(trainable_weights):
        return [layer_weights.shape for layer_weights in trainable_weights]

    @staticmethod
    def flatten(trainable_weights):
        return tf.concat([tf.reshape(layer_weights, [-1]) for layer_weights in trainable_weights], 0)

    @staticmethod
    def convert_to_hessian(second_order_derivatives):
        return tf.transpose(tf.concat(
            [tf.reshape(layer_weights, [layer_weights.shape[0], -1]) for layer_weights in second_order_derivatives], 1))

    def restore(self, flattened_trainable_weights):
        restored_list = []
        first_to_recover = 0
        for shape in self.shapes:
            num_elements = shape.num_elements()
            restored_list.append(
                tf.reshape(flattened_trainable_weights[first_to_recover:first_to_recover + num_elements], shape))
            first_to_recover = first_to_recover + num_elements
        return restored_list


def flip_dictionary(dictionary):
    return {value: key for key, value in dictionary.items()}


def create_dir_if_not_exists(file_path):
    Path(os.path.dirname(file_path)).mkdir(parents=True, exist_ok=True)


def initialize_csv_file_if_empty_or_non_existing(file_path, header):
    if not os.path.isfile(file_path) or os.path.getsize(file_path) == 0:
        with open(file_path, "w+") as policy_log:
            policy_log.write(header)


def flatten_list(li):
    return [e for e in li]


def get_config(path):
    with open(path) as config_file:
        config = json.load(config_file)
    return config


def create_random_integer_symmetric_matrix(low, high, size):
    m = np.tril(np.random.randint(low, high, size), -1)
    return m + m.T


def random_one_hot(n_classes, size):
    # src: https://stackoverflow.com/questions/45093615/random-one-hot-matrix-in-numpy
    return np.eye(n_classes)[np.random.choice(n_classes, size)]


def print_walk_error(error):
    print(error)


class PickableDefaultDict(dict):
    """
    adapted from: https://teastares.github.io/the-conflict-between-defaultdict-and-pickle
    """

    def __init__(self, default_item, nested=False):
        super().__init__()
        self.default_item = default_item
        self.nested = nested

    def set_default_item(self, default_item):
        self.default_item = default_item

    def __missing__(self, key):
        if self.nested:
            self[key] = PickableDefaultDict([])
        else:
            self[key] = deepcopy(self.default_item)
        return self[key]
