import unittest
from utils.utils import TrainableWeightsReshaper
import tensorflow.keras as keras
import tensorflow as tf
from keras.utils.layer_utils import count_params


class TestTrainableWeightsReshaper(unittest.TestCase):

    def setUp(self):
        inputs = keras.layers.Input(shape=(5, ))
        x = keras.layers.Dense(20)(inputs)
        x = keras.layers.Dense(10)(x)
        self.test_model = keras.Model(inputs, x)
        self.reshaper = TrainableWeightsReshaper(self.test_model.trainable_weights)

    def test_initialization(self):
        self.assertEqual([tf.TensorShape([5, 20]), tf.TensorShape([20]), tf.TensorShape([20, 10]),
                          tf.TensorShape([10])], self.reshaper.shapes)

    def test_flatten(self):
        actual = self.reshaper.flatten(self.test_model.trainable_weights)
        self.assertEqual(count_params(self.test_model.trainable_weights), len(actual))

    def test_restore(self):
        flattened = self.reshaper.flatten(self.test_model.trainable_weights)
        restored = self.reshaper.restore(flattened)
        for i in range(len(restored)):
            tf.assert_equal(restored[i], self.test_model.trainable_weights[i])


if __name__ == '__main__':
    unittest.main()
